SystemOrganization addCategory: #SeasideAsync!

!WAStandardScripts methodsFor: '*seasideasync' stamp: 'mb 10/31/2005 09:05'!
liveUpdate
	^ '
function liveUpdaterUri(uri)
{
	return liveUpdater(function() { return uri; },false);
}

function addParameter(uri, key, value)
{
	var separator = "?";
	if (uri.indexOf("?") >= 0)
		separator = "&";
	return uri + separator + key + "=" + escape(value);
}

function liveUpdater(uriFunc, ret)
{
	var request = false;
	function update()
	{
		if (request && request.readyState < 4)
			request.abort();
		request = newHttpRequest();
		if (!!request)
			return;
		request.onreadystatechange = processRequestChange;
		// Resolve xml entity before issuing the request
		var uri = addParameter(uriFunc(), "timestamp", (new Date()).getTime().toString());
		var regex = new RegExp("&amp;", "g");
		uri = uri.replace(regex,"&");
		request.open("GET", uri);
		request.send(null);
		return ret;
	}

	function processRequestChange()
	{
		if (request.readyState == 4) {
			var doc = request.responseXML;
			var replacements = [];
			var anyReplacement = false;
// alert( request.responseText);
// alert( request.status);
// alert( request.statusText);
// alert( request.getAllResponseHeaders() );	
// alert( request.responseXML.parseError.errorCode);
// alert( request.responseXML.parseError.reason);
			if (doc !!= null) {
				var elements = doc.getElementsByTagName("*");
				if (elements.length == 0) {
					// IE does not support the above, use an alternative hard-coded technique
					elements = getAllElements(doc)
				}
				// Find html elements with the id= attribute
				for ( var x = 1 ; x <= elements.length ; x++ ) {
					var elem = elements[x-1];
					if (elem.tagName == "body") {
						var body = xmlString(elem);
					}
					var attributes = elem.attributes;
					if (attributes.length > 0) {
						for ( var ax = 1 ; ax <= attributes.length ; ax++ ) {
							var attr = attributes[ax-1];
							var attrName = attr.name;
							if (attrName == "id") {
								// Find the html element in the document that has a matching id and replace it
								var docElement = document.getElementById(attr.value);
								if (docElement) {
									if (!!includes(replacements,elem.parentNode)) {
										// HTML only updated with parent at highest level
										// Resolve IE and Opera issue where <br></br> skips two lines
										var regex = new RegExp("<br></br>", "g");
										var replacement = xmlString(elem).replace(regex,"<br/>");
										docElement.innerHTML = replacement;
										anyReplacement = true;
									}
									replacements[replacements.length] = elem;
								}
							}
						}
					}   
				}
				// Find html <script> elements and execute them
				for ( var x = 1 ; x <= elements.length ; x++ ) {
					var elem = elements[x-1];
					if (elem.tagName == ''script'') {
						eval(elem.childNodes[0].data);
					}
				}
			}
			if (!!anyReplacement) {
				if ( body ) {
					document.body.innerHTML = body;
				} else {
					// Most probable reason for being here : response does not have an xml content
					// For instance : an exception in the live callback
					document.body.innerHTML = request.responseText;
				}
			}
		}
	}
	return update;
}

function liveSelect(id, uri)
{
	function constructUri()
	{
		return addParameter(uri, "s", document.getElementById(id).selectedIndex.toString());
	}
	return liveUpdater(constructUri, false);
}

function liveSearch(id, uri)
{
	function constructUri()
	{
		return addParameter(uri, "s", document.getElementById(id).value);
	}

   	var updater = liveUpdater(constructUri, false);
	var last = "";
	var timeout = false;
  
	function update()
	{
		if (last !!= document.getElementById(id).value)
			updater();
	}
	   
	function start() {
		if (timeout)
			window.clearTimeout(timeout);
		timeout = window.setTimeout(update, 300);
	}

	if (navigator.userAgent.indexOf("Safari") > 0)
		document.getElementById(id).addEventListener("keydown",start,false);
	else if (navigator.product == "Gecko")
		document.getElementById(id).addEventListener("keypress",start,false);
	else
		document.getElementById(id).attachEvent("onkeydown",start);
}

function liveCheck(id, uri)
{
	function constructUri()
	{
		return addParameter(uri, "s", document.getElementById(id).checked);
	}
	return liveUpdater(constructUri);
}

function liveScroll(id, uri)
{
   function constructUri()
   {
         return addParameter(uri, "s", document.getElementById(id).scrollTop.toString());
   }
   
   var updater = liveUpdater(constructUri, false);
   var timeout = false;

    function start() {
	   if (timeout)
	       window.clearTimeout(timeout);
	   
    	   timeout = window.setTimeout(updater, 100);
    }
   return start;
}

function timedUpdate(uriFunc, milliSeconds)
{
	var updater = liveUpdaterUri(uriFunc, false);
	updater();
	var again = "timedUpdate(\""+uriFunc+"\","+milliSeconds+")";
	setTimeout(again, milliSeconds);
}
'! !

!WAStandardScripts methodsFor: '*seasideasync' stamp: 'mb 10/29/2005 11:19'!
liveUpdateUtilities
	^'
// Function below to instantiate an XMLHttpRequest object
// Credits: http://jibbering.com/2002/4/httprequest.html
// Credits: http://www.toutjavascript.com/savoir/xmlhttprequest.php3
function newHttpRequest()
{
	var httpRequest = false;
	if(window.XMLHttpRequest)
		// Firefox, Mozilla, Safari
		httpRequest = new XMLHttpRequest();
	else if(window.ActiveXObject) 
		// Internet Explorer
		{	try {  
 				httpRequest = new ActiveXObject("Microsoft.XMLHTTP");
			} catch (e) {
  				try {
   					httpRequest = new ActiveXObject("Msxml2.XMLHTTP");
				} catch (E) {
					httpRequest = false;
 	 			}
			}
		}
	return httpRequest;
}

// Functions below to rebuild the xml string of an element
function xmlString (element) {
	var xml = "";
	for ( var x = 1 ; x <= element.childNodes.length ;  x++ ) {
		var node = element.childNodes[x-1];
		xml += outerXmlString(node);
	}
	return xml;
}
function outerXmlString (element) {
	if (!!element.tagName)
		return element.data;
	var xml = "<" + element.tagName;
	for ( var x = 1 ; x <= element.attributes.length ;  x++ ) {
		var attr = element.attributes[x-1];
		xml += " " + attr.name + "=" + ''"'' + attr.value + ''"'';
	}
	xml += ">";
	for ( var x = 1 ; x <= element.childNodes.length ;  x++ ) {
		var node = element.childNodes[x-1];
		xml += outerXmlString(node);
	}
	xml += "</" + element.tagName + ">";
	return xml;
}

// Function below to get stable equivalent of document.getElementsByTagName("*")
// Credits: http://www.siteexperts.com/tips/contents/ts16/page3.asp
function getAllElements(document)
{
	var elements = [];
	getElements(elements,document.childNodes[0])
	return elements;   
}

// Function below to recursively and deeply collect all sub-elements of an element
// Credits: http://www.siteexperts.com/tips/contents/ts16/page3.asp
function getElements(collection, element) {
	if (element == null) return;
	for ( var x = 1 ; x <= element.childNodes.length ; x++ ) {
		var node = element.childNodes[x-1];
		if (node.nodeType == 1) {
			collection[collection.length] = node[x-1];
				getElements(collection,element);
		}
	}
}

// Function below to detect if an element is part of an array
function includes(collection, element) {
	for ( var x = 1 ; x <= collection.length ; x++ ) {
		if (collection[x-1] == element) return true;
	}
	return false;
}

// Get the text of the body element
function bodyText(doc)
{
	var elements = doc.getElementsByTagName("*");
	if (elements.length == 0) {
		// IE does not support the above, use an alternative hard-coded technique
		elements = getAllElements(doc)
	}
	// Find html elements with the id= attribute
	for ( var x = 1 ; x <= elements.length ; x++ ) {
		var elem = elements[x-1];
		if (elem.tagName == "BODY") {
			var body = xmlString(elem);
		}
	}
	return body;	
}

'! !

WAComponent subclass: #WACanvasLiveRequestTest
	instanceVariableNames: 'input1 input2 selection count bool'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SeasideAsync'!

!WACanvasLiveRequestTest methodsFor: 'as yet unclassified' stamp: 'mb 10/30/2005 18:36'!
renderContentOn: html

	html table:
		[html tableRow:
			[html tableData:
				[self renderInput1On: html.
				self renderInput2On: html.
				self renderListOn: html.
				self renderCheckBoxOn: html.
				self renderAnchorOn: html].
			html tableData:
				[html heading: 'Forward' level: 3.
				html div id: 'forward'; with: ''.
				html heading: 'Backward' level: 3.
				html div id: 'backward'; with: ''.
				html heading: 'Selection' level: 3.
				html span id: 'selection'; with: ''.
				html heading: 'Checkbox' level: 3.
				html span id: 'checkbox'; with: ''.
				html heading: 'Time' level: 3.
				html span id: 'time'; with: ''].
			]
		]

! !

!WACanvasLiveRequestTest class methodsFor: 'as yet unclassified' stamp: 'mb 10/29/2005 11:39'!
canBeRoot
	^ true! !

!WACanvasLiveRequestTest class methodsFor: 'as yet unclassified' stamp: 'mb 10/30/2005 18:51'!
initialize
	self registerAsApplication: 'LiveCanvasTests'.! !

!WACanvasLiveRequestTest methodsFor: 'as yet unclassified' stamp: 'mb 10/30/2005 18:23'!
renderCheckBoxOn: html

	html form:
		[html checkbox
				value: bool; 
				callback: [:v | bool := v];
				liveCallback: 
					[:item :h |
					bool := item.
					"Update multiple areas of the document using one live callback"
					h span id: 'checkbox'; with: (bool ifTrue: ['checked'] ifFalse: ['unchecked']).
					h span id: 'chk'; with: (bool ifTrue: ['checked'] ifFalse: ['unchecked']).
					].
			
		html space.
		html span id: 'chk'; with: (bool ifTrue: ['checked'] ifFalse: ['unchecked']).
		].
	! !

!WACanvasLiveRequestTest methodsFor: 'as yet unclassified' stamp: 'dwj 6/9/2005 15:59'!
rendererClass
	^ WARenderCanvas! !

!WACanvasLiveRequestTest methodsFor: 'as yet unclassified' stamp: 'mb 10/30/2005 18:22'!
renderInput1On: html

	html form:
		[html textInput
				value: input1; 
				callback: [:v | input1 := v];
				liveCallback: 
					[:text :h |
					input1 := text. 
					"Update multiple areas of the document using one live callback"
					h div id: 'forward'; with:
						[h unorderedList:
							[input1 do:
								[:c | 
								h listItem:
									[h anchor 
									callback: [self inform: c displayString];
									text: c displayString]]]].
					h span id: 'fwd'; with:
						[h text: 'Value entered= ' , input1.
						h script: 'alert("javascript in a live callback ' , input1 , ' ");']
					].
		html space.
		html span id: 'fwd'; with:'Value entered= ' , input1.
		html break.
		html text: 'Above: Javascript attached= alert("Javascript in a live callback xxx ");'
		].
! !

!WACanvasLiveRequestTest methodsFor: 'as yet unclassified' stamp: 'mb 10/30/2005 18:27'!
renderAnchorOn: html

	html anchor
		liveCallback:
			[:h | 
			h span id: 'timestamp'; with: TimeStamp now.
			h span id: 'time'; with: Time now];
		text: 'Show time';
		callback: [self error: 'Should not come here'].
	html space.
	html span id: 'timestamp'; with: ''.
! !

!WACanvasLiveRequestTest methodsFor: 'as yet unclassified' stamp: 'mb 10/30/2005 18:21'!
renderListOn: html.

	html form:
		[html select 
				list: #(a b c); 
				selected: selection; 
				labels: [:ea | ea displayString];
				callback: [:v | selection := v];
				liveCallback:
					[:item :h |
					selection := item. 
					"Update multiple areas of the document using one live callback"
					h span id: 'selection'; with: selection.
					h span id: 'sel'; with: 'Value selected= ' , selection displayString
					].
		html space.
		html span id: 'sel'; with: 'Value selected= ' , selection displayString
		].
! !

!WACanvasLiveRequestTest methodsFor: 'as yet unclassified' stamp: 'mb 10/30/2005 18:12'!
initialize
	count _ 0.
	bool _ false.
	input1 _ String new.
	input2 _ String new.
! !

!WACanvasLiveRequestTest methodsFor: 'as yet unclassified' stamp: 'mb 10/30/2005 18:22'!
renderInput2On: html

	html form:
		[html textInput
				value: input2;
				callback: [:v | input2 := v];
				liveCallback: 
					[:text :h |
					input2 := text.
					h div id: 'backward'; with: 
						[h unorderedList: 
							[input2 reverse do: 
								[:c |
								h listItem: 
									[h anchor 
										callback: [self inform: c displayString];
										text: c displayString]]]].
					h span id: 'bwd'; with: 'Value entered= ' , input2
					].
		html space.
		html span id: 'bwd'; with: 'Value entered= ' , input2
		].
! !

WAComponent subclass: #WALiveRequestTest
	instanceVariableNames: 'input1 input2 selection count bool country province city log'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SeasideAsync'!

!WALiveRequestTest methodsFor: 'as yet unclassified' stamp: 'mb 10/30/2005 18:48'!
province: anObject
	province := anObject.
	self city: self cityNames first.

! !

!WALiveRequestTest methodsFor: 'as yet unclassified' stamp: 'mb 10/30/2005 18:46'!
city: anObject
	city := anObject.
! !

!WALiveRequestTest methodsFor: 'as yet unclassified' stamp: 'mb 10/30/2005 18:39'!
renderAnchorOn: html

	html anchorWithAction: 
				[self error: 'Should not come here']
			liveAction:
				[:h | 
				h spanNamed: 'timestamp' with: TimeStamp now.
				h spanNamed: 'time' with: Time now]
			text: 'Show time'.
	html space.
	html spanNamed: 'timestamp' with: ''.
! !

!WALiveRequestTest methodsFor: 'as yet unclassified' stamp: 'mb 10/31/2005 09:58'!
renderLogOn: html

	| stream line |
	stream := log contents readStream.
	[stream atEnd] whileFalse:
		[line := stream nextLine.
		html text: (line copyWithoutAll: (Array with: Character cr with: Character lf)); break].

	
! !

!WALiveRequestTest methodsFor: 'as yet unclassified' stamp: 'mb 10/30/2005 18:44'!
renderCitiesOn: html

	html spanNamed: 'citySelector' with:
		[html 
			selectFromList: self cityNames
			selected: city
			callback:[:f | city := f ]						
			labels: [:ea |  ea ]]		
! !

!WALiveRequestTest methodsFor: 'as yet unclassified' stamp: 'mb 10/31/2005 07:46'!
initialize
	count _ 0.
	bool  _ false.
	input1 _ String new.
	input2 _ String new.
	self country: 'Argentina'.
	log := String new writeStream.



! !

!WALiveRequestTest methodsFor: 'as yet unclassified' stamp: 'mb 10/30/2005 18:10'!
renderCheckBoxOn: html
	
	html form:
		[html checkboxWithValue: bool 
				callback: [:v | bool := v ] 
				liveCallback: 
					[:item :h |
					bool := item.
					"Update multiple areas of the document using one live callback"
					h spanNamed: 'checkbox' with: (bool ifTrue: ['checked'] ifFalse: ['unchecked']).
					h spanNamed: 'chk' with: (bool ifTrue: ['checked'] ifFalse: ['unchecked']).
					].
		html space.
		html spanNamed: 'chk' with: (bool ifTrue: ['checked'] ifFalse: ['unchecked']).
		].
! !

!WALiveRequestTest methodsFor: 'as yet unclassified' stamp: 'mb 10/30/2005 18:07'!
renderInput1On: html

	html form:
		[html textInputWithValue: input1
				callback: [:v | input1 := v]
				liveCallback:
					[:text :h |
					input1 := text.
					"Update multiple areas of the document using one live callback"
					h divNamed: 'forward' with:
						[h list: input1 do:
							[:c |
							h anchorWithAction: [self inform: c asString] text: c asString]].
					h spanNamed: 'fwd' with:
						[h text: 'Value entered= ' , input1.
						h script: 'alert("Javascript in a live callback ' , input1 , ' ");'].
					].
		html space.
		html spanNamed: 'fwd' with: 'Value entered= ' , input1.
		html break.
		html text: 'Above: Javascript attached= alert("javascript in a live callback xxx ");'.		
		].
! !

!WALiveRequestTest methodsFor: 'as yet unclassified' stamp: 'mb 10/30/2005 18:47'!
renderProvincesOn: html

	html spanNamed: 'provinceSelector' with:
		[html
			selectFromList:  self provinceNames
			selected: province
			callback: [:f | province := f] 
			labels: [:ea |  ea ]
			liveCallback: 
				[:anObject :h | 
				province := anObject. 
				city := self cityNames first.
				self renderCitiesOn: h]].
	self renderCitiesOn: html.


! !

!WALiveRequestTest class methodsFor: 'as yet unclassified' stamp: 'mb 10/29/2005 11:39'!
canBeRoot
	^ true! !

!WALiveRequestTest methodsFor: 'as yet unclassified' stamp: 'mb 10/30/2005 18:07'!
renderInput2On: html

	html form:
		[html textInputWithValue: input2 
				callback: [:v | input2 := v]
				liveCallback:
					[:text :h |
					input2 := text.
					"Update multiple areas of the document using one live callback"
					h divNamed: 'backward' with:
						[h list: input2 reverse do:
							[:c |
							h anchorWithAction: [self inform: c asString] text: c asString]].
					h spanNamed: 'bwd' with: 'Value entered= ' , input2.
					].
		html space.
		html spanNamed: 'bwd' with: 'Value entered= ' , input2.
		].
! !

!WALiveRequestTest methodsFor: 'as yet unclassified' stamp: 'mb 10/30/2005 18:08'!
renderListOn: html

	html form:
		[html selectFromList: #(a b c) 
				selected: selection 
				callback: [:v | selection := v]
				labels: [:ea | ea asString]
				liveCallback:
					[:item :h |
					selection := item.
					"Update multiple areas of the document using one live callback"
					h spanNamed: 'selection' with: item.
					h spanNamed: 'sel' with: 'Value selected= ' , selection displayString.
					].
		html space.
		html spanNamed: 'sel' with: 'Value selected= ' , selection displayString.
		].
! !

!WALiveRequestTest methodsFor: 'as yet unclassified' stamp: 'mb 10/30/2005 18:48'!
cityNames
	province = 'Oranjestad'
		ifTrue: [^#( 'Casibari' 'Bubali')	].
	province = 'Santa Cruz'
		ifTrue: [^#( 'Arikok' 'Yamonta')	].
	province = 'Buenos Aires'
		ifTrue: [^#('La Plata' 'Azul')].
	province = 'Cordoba'
		ifTrue: [^#('Alta Gracia' 'La Falda ')].
	^#()


! !

!WALiveRequestTest methodsFor: 'as yet unclassified' stamp: 'mb 10/30/2005 18:48'!
country: anObject
	country := anObject. 
	self province: self provinceNames first. 
! !

!WALiveRequestTest methodsFor: 'as yet unclassified' stamp: 'mb 10/30/2005 18:48'!
provinceNames
	country = 'Aruba'
		ifTrue: [^#('Oranjestad' 'Santa Cruz')].
	country = 'Argentina'
		ifTrue: [^#( 'Buenos Aires' 'Cordoba')].
	^#()


! !

!WALiveRequestTest methodsFor: 'as yet unclassified' stamp: 'mb 10/30/2005 18:46'!
renderCountriesOn: html 

	html
		selectFromList: #( 'Argentina' 'Aruba' )
		selected: country
		callback: [:f | country := f] 
		labels: [:ea |  ea ]
		liveCallback: 
			[:anObject :h | 
			country := anObject. 
			province := self provinceNames first. 
			city := self cityNames first.
			self renderProvincesOn: h].

	self renderProvincesOn: html.
! !

!WALiveRequestTest methodsFor: 'as yet unclassified' stamp: 'mb 10/31/2005 09:06'!
renderContentOn: html

	| liveUpdateBlock |
	liveUpdateBlock _ [ :text :h |
		log nextPutAll: 'Timed log at ' , Time now printString; cr; nextPut: Character lf.
		h text: Time now printString ].
	
	html table: [
		html tableRowWith: [
					
			html table: [
		
				html tableRowWith: [
					html text: 'Current Time in default div:']
				with: [
					html refreshDivWith: liveUpdateBlock every: 2 seconds ].
		
				html tableRowWith: [
					html text: 'Current Time in another div:']
				with: [
					html refreshDivNamed: 'mylivediv' with: liveUpdateBlock every: 3 seconds ].
			
				html tableRowWith: [
					html text: 'Current Time in default span:']
				with: [
					html refreshSpanWith: liveUpdateBlock every: 5 seconds. ].
			].

			
			self renderInput1On: html.
			self renderInput2On: html.
			self renderListOn: html.
			self renderCheckBoxOn: html.
			self renderAnchorOn: html.

			html break.
			html anchorWithAction: 
						[self error: 'Should not come here'] 
					liveAction: 
						[:h | h spanNamed: 'time' with: 1/0] 
					text: 'Live error'.
			html break; break.
			html scrollbarWithHeight: 100 size: 1000 liveCallback: [:n :h | h spanNamed: 'scroll' with: n]	
			] 
		with: [
			html heading: 'Forward' level: 3.
			html divNamed: 'forward' with: ''.
			html heading: 'Backward' level: 3.
			html divNamed: 'backward' with: ''.
			html heading: 'Selection' level: 3.
			html spanNamed: 'selection' with: ''.
			html heading: 'Check box' level: 3.
			html spanNamed: 'checkbox' with: (bool ifTrue: ['checked'] ifFalse: ['unchecked']).
			html heading: 'Time' level: 3.
			html spanNamed: 'time' with: ''.
			html heading: 'Scrollbar' level: 3.
			html spanNamed: 'scroll' with: ''.
			]
	].

	html heading: 'Nested live callbacks' level: 3.
	self renderCountriesOn: html.

	html heading: 'Refreshed log' level: 3.
	html
		divNamed: 'log'
		with: [self renderLogOn: html]
		refreshWith:
			[ :text :h |
			html text: text; break.
			self renderLogOn: h]
	 	every: 5 seconds ! !

!WALiveRequestTest class methodsFor: 'as yet unclassified' stamp: 'mb 10/30/2005 18:51'!
initialize
	self registerAsApplication: 'LiveTests'.! !

!WARenderCanvas methodsFor: '*seasideasync' stamp: 'mb 10/30/2005 17:07'!
urlForLiveRequest: aBlock
	| request text ts response renderer |
	request _ ResponseContinuation currentDo: [ :cc | 
		^ WACurrentSession value 
			actionUrlForContinuation: cc ].
	text _ request at: 's' ifAbsent: [].
	ts _ request at: 'timestamp'.	"Unused, but documented like that"
	response _ WAResponse new.
	response contentType: 'text/xml; charset=utf-8'.
	renderer _ self class 
		context: self context copy
		callbacks: self callbacks.
	document _ WAHtmlStreamDocument new 
		stream: response stream.
	renderer context document: document.
	aBlock value: text value: renderer.
	document close.
	WACurrentSession value returnResponse: response! !

!WAHtmlRenderer methodsFor: '*seasideasync' stamp: 'mb 10/29/2005 11:14'!
scrollbarWithHeight: heightNumber size: sizeNumber liveCallback: liveBlock
	| id uri |
	id _ self ensureId.
	self attributeAt: 'style' 
		put: 'position: relative; left: -3px; width: 19px; overflow: auto; height: '
			, heightNumber asString
			, 'px'.
	self div:
		[self attributeAt: 'style' put: 'width: 1px; height: ', sizeNumber asString, 'px'.
		self div: ''].
	uri _ self urlForLiveRequest: [:t :h | liveBlock value: t asInteger value: h].
	self 
		script: 'document.getElementById('
				, id printString
				, ').onscroll = liveScroll('
				, id printString
				, ', '
				, (self class encode: uri asString) printString
				, ')'! !

!WAHtmlRenderer methodsFor: '*seasideasync' stamp: 'mb 10/31/2005 08:42'!
refreshElementNamed: elementName createElementWith: aSymbol with: aBlock refreshWith: liveBlock every: aDuration
	"Private method to creates an element using aSymbol that executes and displays the result 
	of liveBlock in intervals of aDuration. The interval starts at the end of execution: 
	if liveBlock is time consuming, the next execution will happen aDuration later.
	No overlapping refreshes will occur.
	See #refreshDivWith:every: and refreshSpanWith:"
	
	| uri |
	self perform: aSymbol with: elementName with: aBlock.
	uri := self urlForLiveRequest:
			[ :text :h | 
			h divNamed: elementName with: [liveBlock value: text value: h]].
	self 
		script: 'timedUpdate('
				, (self class encode: uri asString) printString
				, ', ' 
				, aDuration asMilliSeconds printString
				, ')'
! !

!WAHtmlRenderer methodsFor: '*seasideasync' stamp: 'mb 10/31/2005 08:33'!
divNamed: divName with: aBlock refreshWith: liveBlock every: aDuration

	self
		refreshElementNamed: divName
		createElementWith: #divNamed:with:
		with: aBlock
		refreshWith: liveBlock
		every: aDuration
! !

!WAHtmlRenderer methodsFor: '*seasideasync' stamp: 'mb 10/31/2005 08:37'!
refreshSpanNamed: spanName with: liveBlock every: aDuration

	self
		spanNamed: spanName 
		with: [] 
		refreshWith: liveBlock 
		every: aDuration! !

!WAHtmlRenderer methodsFor: '*seasideasync' stamp: 'mb 10/29/2005 11:12'!
anchorWithAction: actionBlock liveAction: liveBlock text: aString
	| uri id |
	id _ self ensureId.
	self anchorWithAction: actionBlock text: aString.
	uri _ self urlForLiveRequest: [:t :h | liveBlock value: h].
	self 
		script: 'document.getElementById('
				, id printString
				, ').onclick = liveUpdaterUri('
				, (self class encode: uri asString) printString
				, ')'! !

!WAHtmlRenderer methodsFor: '*seasideasync' stamp: 'mb 10/31/2005 08:38'!
refreshSpanWith: liveBlock every: aDuration
	"Convenience method if you need only one automatically refreshing span on a page."

	self
		refreshSpanNamed: 'default-live-update-span'
		with: liveBlock
		every: aDuration
! !

!WAHtmlRenderer methodsFor: '*seasideasync' stamp: 'mbany 4/14/2005 12:34'!
selectFromList: aCollection selected: anObject callback: aBlock labels: labelBlock other: otherBlock label: aString
	| id other  |
	id _ 'id', self context advanceKey displayString.
	other _ (self context nextKey asNumber + aCollection size + 1) displayString.
	self attributes onChange: 'if(this.value == ', other printString, ') {document.getElementById(', id printString, ').style.display = "inline"; document.getElementById(', id printString, ').focus()} else {document.getElementById(', id printString, ').style.display = "none"}'.
	self
		selectFromList: (aCollection asArray copyWith: aString)
		selected: anObject
		callback: [:v | v = aString ifFalse: [aBlock value: v]]
		labels: [:ea | ea = aString ifTrue: [aString] ifFalse: [labelBlock value: ea]].
	self space.
	self cssId: id.
	self attributes style: 'display: none'.
	self textInputWithValue: nil callback: [:v | v ifNotNil: [otherBlock value: v]]! !

!WAHtmlRenderer methodsFor: '*seasideasync' stamp: 'mb 10/31/2005 08:35'!
refreshDivNamed: divName with: liveBlock every: aDuration

	self
		divNamed: divName
		with: []
		refreshWith: liveBlock
		every: aDuration! !

!WAHtmlRenderer methodsFor: '*seasideasync' stamp: 'mb 10/30/2005 17:07'!
urlForLiveRequest: aBlock
	| request text ts response renderer document |
	request _ ResponseContinuation currentDo: [ :cc | 
		^ WACurrentSession value 
			actionUrlForContinuation: cc ].
	text _ request at: 's' ifAbsent: [].
	ts _ request at: 'timestamp'.	"Unused, but documented like that"
	response _ WAResponse new.
	response contentType: 'text/xml; charset=utf-8'.
	renderer _ self class
		context: self context copy
		callbacks: self callbacks.
	document _ WAHtmlStreamDocument new 
		stream: response stream.
	renderer context document: document.
	aBlock value: text value: renderer.
	document close.
	WACurrentSession value returnResponse: response! !

!WAHtmlRenderer methodsFor: '*seasideasync' stamp: 'mb 10/30/2005 17:41'!
textInputWithCallback: callbackBlock liveCallback: liveBlock
	self textInputWithValue: '' callback: callbackBlock liveCallback: liveBlock! !

!WAHtmlRenderer methodsFor: '*seasideasync' stamp: 'mb 10/31/2005 08:34'!
spanNamed: spanName with: aBlock refreshWith: liveBlock every: aDuration

	self
		refreshElementNamed: spanName
		createElementWith: #spanNamed:with:
		with: aBlock
		refreshWith: liveBlock
		every: aDuration
! !

!WAHtmlRenderer methodsFor: '*seasideasync' stamp: 'mb 10/29/2005 11:15'!
selectFromList: aCollection selected: anObject callback: callbackBlock labels: labelsBlock liveCallback: liveBlock
	| uri id items |
	id _ self ensureId.
	items _ aCollection asArray.
	self selectFromList: items selected: anObject callback: callbackBlock labels: labelsBlock.
	uri _ self urlForLiveRequest: [:index :h | liveBlock value: (items at: index asInteger + 1 ifAbsent: []) value: h].
	self 
		script: 'document.getElementById('
				, id printString
				, ').onchange = liveSelect('
				, id printString
				, ','
				, (self class encode: uri asString) printString
				, ')'
! !

!WAHtmlRenderer methodsFor: '*seasideasync' stamp: 'mb 10/30/2005 17:39'!
textInputWithValue: value callback: callbackBlock liveCallback: liveBlock
	| uri id |
	id _ self ensureId.
	self textInputWithValue: value callback: callbackBlock.
	uri _ self urlForLiveRequest: liveBlock.
	self 
		script: 'liveSearch('
				, id printString
				, ','
				, (self class encode: uri asString) printString
				, ')'.
	! !

!WAHtmlRenderer methodsFor: '*seasideasync' stamp: 'mb 10/29/2005 11:12'!
checkboxWithValue: aBoolean callback: callbackBlock liveCallback: liveBlock 
	| uri id |
	id _ self ensureId.
	self checkboxWithValue: aBoolean callback: callbackBlock.
	uri _ self urlForLiveRequest: [:v :h | liveBlock value: v = 'true' value: h].
	self 
		script: 'document.getElementById(' 
				, id printString 
				, ').onclick = liveCheck(' 
				, id printString 
				, ',' 
				, (self class encode: uri asString) printString 
				, ')'! !

!WAHtmlRenderer methodsFor: '*seasideasync' stamp: 'mb 10/31/2005 08:36'!
refreshDivWith: liveBlock every: aDuration
	"Convenience method if you need only one automatically refreshing div on a page."

	self
		refreshDivNamed: 'default-live-update-div'
		with: liveBlock
		every: aDuration
! !

!WAAnchorTag methodsFor: '*seasideasync' stamp: 'mb 10/29/2005 11:50'!
liveCallback: liveBlock
	| uri id |
	id := self ensureId.
	uri := canvas urlForLiveRequest: [:t :h | liveBlock value: h].
	self 
		onClick: 'liveUpdaterUri('
				, (canvas encode: uri asString) printString
				, ')()'! !

!WAHtmlCanvas methodsFor: '*seasideasync' stamp: 'mb 10/29/2005 11:11'!
space
	self html: '&#160;'! !

!WACheckboxTag methodsFor: '*seasideasync' stamp: 'mb 10/30/2005 17:14'!
liveCallback: liveBlock
	| uri id |
	id := self ensureId.
	uri := canvas urlForLiveRequest: [:v :h | liveBlock value: v = 'true' value: h].
	canvas 
		script: 'document.getElementById('
				, id printString
				, ').onclick = liveCheck(' 
				, id printString
				, ','
				, (canvas encode: uri asString) printString
				, ')'
! !

!WASelectTag methodsFor: '*seasideasync' stamp: 'mb 10/29/2005 11:51'!
liveCallback: liveBlock
	| uri id |
	id := self ensureId.
	uri := canvas urlForLiveRequest: [:index :h | liveBlock value: (list at: index asInteger + 1 ifAbsent: []) value: h].
	canvas 
		script: 'document.getElementById('
				, id printString
				, ').onchange = liveSelect('
				, id printString
				, ','
				, (canvas encode: uri asString) printString
				, ')'
! !

!WATextInputTag methodsFor: '*seasideasync' stamp: 'mb 10/29/2005 11:51'!
liveCallback: aBlock
	| uri id |
	id := self ensureId.
	uri := canvas urlForLiveRequest: aBlock.
	canvas
		script: 'liveSearch('
				, id printString
				, ','
				, (canvas encode: uri asString) printString
				, ')'.! !

!WAAbstractHtmlBuilder methodsFor: '*seasideasync' stamp: 'mb 10/29/2005 11:09'!
space
	self html: '&#160;'! !
WACanvasLiveRequestTest initialize!
WALiveRequestTest initialize!
